@@ -3,7 +3,7 @@
 #' @description `r lifecycle::badge('experimental')`
 #'  Add restrictions, like a bonus-malus structure, on the risk
 #'  factors used in the model. `restrict_coef()` must always be followed
-#'  by `refit_glm()`.
+#'  by `update_glm()`.
 #'
 #' @author Martin Haringa
 #'
@@ -11,18 +11,20 @@
 #'   the severity model, it is more appropriate to impose the restrictions
 #'   on the premium model. This can be achieved by calculating the pure
 #'   premium for each record (i.e. expected number of claims times the expected
-#'   claim amount), then fitting an "unrestricted" Gamma GLM to the pure premium,
-#'   and then imposing the restrictions in a final "restricted" Gamma GLM.
+#'   claim amount), then fitting an "unrestricted" Gamma GLM to the pure
+#'   premium,and then imposing the restrictions in a final "restricted" Gamma
+#'   GLM.
 #'
 #' @param model object of class glm/restricted
-#' @param restrictions data.frame with two columns containing restricted data. The first
-#'   column, with the name of the risk factor as column name, must contain the
-#'   levels of the risk factor. The second column must contain the restricted
-#'   coefficients.
+#' @param restrictions data.frame with two columns containing restricted data.
+#'   The first column, with the name of the risk factor as column name, must
+#'   contain the levels of the risk factor. The second column must contain the
+#'   restricted coefficients.
 #'
-#' @family refit_glm
+#' @family update_glm
 #' @family autoplot.restricted
-#' @seealso [refit_glm()] for refitting the restricted model, and [autoplot.restricted()].
+#' @seealso [update_glm()] for refitting the restricted model,
+#' and [autoplot.restricted()].
 #'
 #' @return Object of class restricted.
 #'
@@ -34,7 +36,8 @@
 #' library(dplyr)
 #' freq <- glm(nclaims ~ bm + zip, offset = log(exposure), family = poisson(),
 #'              data = MTPL)
-#' sev <- glm(amount ~ bm + zip, weights = nclaims, family = Gamma(link = "log"),
+#' sev <- glm(amount ~ bm + zip, weights = nclaims,
+#'             family = Gamma(link = "log"),
 #'             data = MTPL %>% filter(amount > 0))
 #'
 #' # Add predictions for freq and sev to data, and calculate premium
@@ -52,7 +55,7 @@
 #' # Fit restricted model
 #' burn_rst <- burn %>%
 #'   restrict_coef(., zip_df) %>%
-#'   refit_glm()
+#'   update_glm()
 #'
 #' # Show rating factors
 #' rating_factors(burn_rst)
@@ -69,10 +72,14 @@ restrict_coef <- function(model, restrictions){
     model_call <- model$call
     model_out <- model
 
-    rfdf <- rating_factors(model)$df
+    rfdf <- rating_factors1(model)
     rst_lst <- list(restrictions)
     names(rst_lst) <- names(restrictions[1])
     restricted_df <- restrict_df(restrictions)
+    new_col_nm <- NULL
+    old_col_nm <- NULL
+    mgd_rst <- NULL
+    mgd_smt <- NULL
   }
 
   if ( inherits(model, c("smooth", "restricted")) ){
@@ -87,6 +94,10 @@ restrict_coef <- function(model, restrictions){
     rst_lst <- model$restrictions_lst
     rst_lst[[names(restrictions)[1]]] <- restrictions
     restricted_df <- restrict_df(restrictions)
+    new_col_nm <- model$new_col_nm
+    old_col_nm <- model$old_col_nm
+    mgd_rst <- model$mgd_rst
+    mgd_smt <- model$mgd_smt
   }
 
   if ( inherits(model, "restricted") ){
@@ -101,6 +112,16 @@ restrict_coef <- function(model, restrictions){
   fm_add <- update_formula_add(offset_term, fm_remove, names(restrictions)[2])
   df_restricted <- add_restrictions_df(df_new, restrictions)
 
+  nrst <- unique(setdiff(names(restrictions), unique(rfdf$risk_factor)))
+  orst <- unique(setdiff(names(restrictions), new_col_nm))
+  mgd_rst <- append(mgd_rst, list(unique(c(orst, nrst))))
+
+  new_col_nm <- unique(append(new_col_nm,
+                              setdiff(names(restrictions),
+                                      unique(rfdf$risk_factor))))
+  old_col_nm <- unique(append(old_col_nm, setdiff(names(restrictions),
+                                                  new_col_nm)))
+
   rt <- list(formula_restricted = fm_add[[1]],
              formula_removed = fm_remove,
              data_restricted = df_restricted,
@@ -110,17 +131,22 @@ restrict_coef <- function(model, restrictions){
              restrictions_lst = rst_lst,
              rf_restricted_df = restricted_df,
              model_call = model_call,
-             model_out = model_out)
+             model_out = model_out,
+             new_col_nm = new_col_nm,
+             old_col_nm = old_col_nm,
+             mgd_rst = mgd_rst,
+             mgd_smt = mgd_smt)
   attr(rt, "class") <- "restricted"
   invisible(rt)
 }
 
 
+
 #' Smooth coefficients in the model
 #'
 #' @description `r lifecycle::badge('experimental')`
 #'  Apply smoothing on the risk factors used in the model. `smooth_coef()`
-#'  must always be followed by `refit_glm()`.
+#'  must always be followed by `update_glm()`.
 #'
 #' @author Martin Haringa
 #'
@@ -128,8 +154,9 @@ restrict_coef <- function(model, restrictions){
 #'   the severity model, it is more appropriate to impose the smoothing
 #'   on the premium model. This can be achieved by calculating the pure
 #'   premium for each record (i.e. expected number of claims times the expected
-#'   claim amount), then fitting an "unrestricted" Gamma GLM to the pure premium,
-#'   and then imposing the restrictions in a final "restricted" Gamma GLM.
+#'   claim amount), then fitting an "unrestricted" Gamma GLM to the pure
+#'   premium, and then imposing the restrictions in a final "restricted"
+#'   Gamma GLM.
 #'
 #' @param model object of class glm/smooth
 #' @param x_cut column name with breaks/cut
@@ -137,9 +164,10 @@ restrict_coef <- function(model, restrictions){
 #' @param degree order of polynomial
 #' @param breaks numerical vector with new clusters for x
 #'
-#' @family refit_glm
+#' @family update_glm
 #' @family autoplot.smooth
-#' @seealso [refit_glm()] for refitting the smoothed model, and [autoplot.smooth()].
+#' @seealso [update_glm()] for refitting the smoothed model,
+#' and [autoplot.smooth()].
 #'
 #' @return Object of class smooth
 #'
@@ -164,10 +192,10 @@ restrict_coef <- function(model, restrictions){
 #'   mutate(across(where(is.factor), ~biggest_reference(., exposure)))
 #'
 #' # Fit frequency and severity model
-#' freq <- glm(nclaims ~ bm + age_policyholder_freq_cat, offset = log(exposure), family = poisson(),
-#'             data = dat)
-#' sev <- glm(amount ~ bm + zip, weights = nclaims, family = Gamma(link = "log"),
-#'            data = dat %>% filter(amount > 0))
+#' freq <- glm(nclaims ~ bm + age_policyholder_freq_cat, offset = log(exposure),
+#'  family = poisson(), data = dat)
+#' sev <- glm(amount ~ bm + zip, weights = nclaims,
+#'  family = Gamma(link = "log"), data = dat %>% filter(amount > 0))
 #'
 #' # Add predictions for freq and sev to data, and calculate premium
 #' premium_df <- dat %>%
@@ -192,7 +220,7 @@ restrict_coef <- function(model, restrictions){
 #'   smooth_coef(x_cut = "age_policyholder_freq_cat",
 #'               x_org = "age_policyholder",
 #'               breaks = seq(18, 95, 5)) %>%
-#'   refit_glm()
+#'   update_glm()
 #'
 #' # Show new rating factors
 #' rating_factors(burn_restricted)
@@ -213,8 +241,12 @@ smooth_coef <- function(model, x_cut, x_org, degree = NULL, breaks = NULL){
     model_call <- model$call
     model_out <- model
 
-    rfdf <- rating_factors(model)$df
+    rfdf <- rating_factors1(model)
     rst_lst <- NULL
+    new_col_nm <- NULL
+    old_col_nm <- NULL
+    mgd_smt <- NULL
+    mgd_rst <- NULL
   }
 
   if ( inherits(model, c("smooth", "restricted")) ){
@@ -227,8 +259,18 @@ smooth_coef <- function(model, x_cut, x_org, degree = NULL, breaks = NULL){
 
     rfdf <- model$rating_factors
     rst_lst <- model$restrictions_lst
+    new_col_nm <- model$new_col_nm
+    old_col_nm <- model$old_col_nm
+    mgd_smt <- model$mgd_smt
+    mgd_rst <- model$mgd_rst
   }
 
+  mgd_smt <- append(mgd_smt, list(c(paste0(x_org, "_smooth"),
+                                    paste0(x_cut, "_smooth"))))
+
+  old_col_nm <- append(old_col_nm, paste0(x_org, "_smooth"))
+  new_col_nm <- append(new_col_nm, paste0(x_cut, "_smooth"))
+
   fm_remove <- update_formula_remove(fm_no_offset, x_cut)
   fm_add <- update_formula_add(offset_term, fm_remove, paste0(x_cut, "_smooth"))
 
@@ -267,7 +309,11 @@ smooth_coef <- function(model, x_cut, x_org, degree = NULL, breaks = NULL){
              restrictions_lst = rst_lst,
              new_rf = df_new_rf,
              degree = degree,
-             model_out = model_out)
+             model_out = model_out,
+             new_col_nm = new_col_nm,
+             old_col_nm = old_col_nm,
+             mgd_rst = mgd_rst,
+             mgd_smt = mgd_smt)
   attr(st, "class") <- "smooth"
   invisible(st)
 }
@@ -319,13 +365,16 @@ print.smooth <- function(x, ...){
 #' @author Martin Haringa
 #'
 #' @importFrom dplyr left_join
-#' @importFrom tidyr pivot_longer
+#' @importFrom data.table melt
+#' @importFrom data.table setDT
+#' @importFrom data.table setDF
 #' @import ggplot2
 #'
 #' @return Object of class ggplot2
 #'
 #' @examples
-#' freq <- glm(nclaims ~ bm + zip, weights = power, family = poisson(), data = MTPL)
+#' freq <- glm(nclaims ~ bm + zip, weights = power, family = poisson(),
+#'  data = MTPL)
 #' zip_df <- data.frame(zip = c(0,1,2,3), zip_rst = c(0.8, 0.9, 1, 1.2))
 #' freq %>%
 #'   restrict_coef(., zip_df) %>%
@@ -342,14 +391,22 @@ autoplot.restricted <- function(object, ...){
   naam_rf <- rf[rf$risk_factor == name,]
   naam_rf <- naam_rf[,2:3]
   names(naam_rst)[names(naam_rst) == name] <- "level"
+
   naam_rf <- matchColClasses(naam_rst, naam_rf)
 
   koppel <- dplyr::left_join(naam_rst, naam_rf, by = "level")
-  koppel <- tidyr::pivot_longer(koppel,
-                                cols = c(names(naam_rst)[2], names(rf)[3]),
-                                names_to = "type",
-                                values_to = "Coef")
+  meas_vars <- c(names(naam_rst)[2], names(rf)[3])
+
+  koppel_dt <- data.table::setDT(koppel)
+  koppel_ldt <- data.table::melt(koppel_dt,
+                                 id.vars = names(koppel_dt)[!names(
+                                   koppel_dt) %in% meas_vars],
+                                 measure.vars = meas_vars,
+                                 variable.name = "type",
+                                 value.name = "Coef")
+  koppel <- data.table::setDF(koppel_ldt)
   koppel$level <- as.factor(koppel$level)
+  koppel$type <- as.character(koppel$type)
 
   koppel$type[koppel$type == names(naam_rst)[2]] <- "restricted"
   koppel$type[koppel$type == names(rf)[3]] <- "unrestricted"
@@ -376,7 +433,6 @@ autoplot.restricted <- function(object, ...){
 #' @author Martin Haringa
 #'
 #' @importFrom dplyr left_join
-#' @importFrom tidyr pivot_longer
 #' @import ggplot2
 #' @importFrom scales ordinal
 #'
@@ -404,28 +460,56 @@ autoplot.smooth <- function(object, ...){
   names(new_line)[names(new_line) == x_name] <- "col1"
 
   ggplot2::ggplot(data = rf2) +
-    ggplot2::geom_segment(ggplot2::aes(x = start_, y = estimate, xend = end_, yend = estimate, color = "Model fit"), group = 1) +
-    ggplot2::geom_segment(data = new, ggplot2::aes(x = breaks_min, y = yhat, xend = breaks_max, yend = yhat, color = "New cluster"), group = 2) +
-    ggplot2::geom_point(data = rf2_start_closed, ggplot2::aes(x = start_, y = estimate), color = "dodgerblue") +
-    ggplot2::geom_point(data = rf2_end_closed, ggplot2::aes(x = end_, y = estimate), color = "dodgerblue") +
-    ggplot2::geom_point(data = rf2_start_open, ggplot2::aes(x = start_, y = estimate), color = "dodgerblue", shape = 21, fill = "white") +
-    ggplot2::geom_point(data = rf2_end_open, ggplot2::aes(x = end_, y = estimate), color = "dodgerblue", shape = 21, fill = "white") +
-    ggplot2::geom_point(data = new_start_closed, ggplot2::aes(x = start_, y = yhat), color = "red") +
-    ggplot2::geom_point(data = new_end_closed, ggplot2::aes(x = end_, y = yhat), color = "red") +
-    ggplot2::geom_point(data = new_start_open, ggplot2::aes(x = start_, y = yhat), color = "red", shape = 21, fill = "white") +
-    ggplot2::geom_point(data = new_end_open, ggplot2::aes(x = end_, y = yhat), color = "red", shape = 21, fill = "white") +
+    ggplot2::geom_segment(ggplot2::aes(x = start_, y = estimate, xend = end_,
+                                       yend = estimate, color = "Model fit"),
+                          group = 1) +
+    ggplot2::geom_segment(data = new, ggplot2::aes(x = breaks_min, y = yhat,
+                                                   xend = breaks_max,
+                                                   yend = yhat,
+                                                   color = "New cluster"),
+                          group = 2) +
+    ggplot2::geom_point(data = rf2_start_closed, ggplot2::aes(x = start_,
+                                                              y = estimate),
+                        color = "dodgerblue") +
+    ggplot2::geom_point(data = rf2_end_closed, ggplot2::aes(x = end_,
+                                                            y = estimate),
+                        color = "dodgerblue") +
+    ggplot2::geom_point(data = rf2_start_open, ggplot2::aes(x = start_,
+                                                            y = estimate),
+                        color = "dodgerblue", shape = 21, fill = "white") +
+    ggplot2::geom_point(data = rf2_end_open, ggplot2::aes(x = end_,
+                                                          y = estimate),
+                        color = "dodgerblue", shape = 21, fill = "white") +
+    ggplot2::geom_point(data = new_start_closed, ggplot2::aes(x = start_,
+                                                              y = yhat),
+                        color = "red") +
+    ggplot2::geom_point(data = new_end_closed, ggplot2::aes(x = end_,
+                                                            y = yhat),
+                        color = "red") +
+    ggplot2::geom_point(data = new_start_open, ggplot2::aes(x = start_,
+                                                            y = yhat),
+                        color = "red", shape = 21, fill = "white") +
+    ggplot2::geom_point(data = new_end_open, ggplot2::aes(x = end_,
+                                                          y = yhat),
+                        color = "red", shape = 21, fill = "white") +
     ggplot2::labs(x = x_name, y = "Estimated coefficient") +
-    ggplot2::geom_line(data = new_line, ggplot2::aes(x = col1, y = yhat, color = "Smooth"), group = 3) +
+    ggplot2::geom_line(data = new_line, ggplot2::aes(x = col1, y = yhat,
+                                                     color = "Smooth"),
+                       group = 3) +
     ggplot2::scale_colour_manual(name = "Risk factor",
-                                 values = c("Model fit" = "dodgerblue", "New cluster" = "red", "Smooth" = "black"),
-                                 labels = c("Model fit", "New cluster", degree_name)) +
+                                 values = c("Model fit" = "dodgerblue",
+                                            "New cluster" = "red",
+                                            "Smooth" = "black"),
+                                 labels = c("Model fit", "New cluster",
+                                            degree_name)) +
     ggplot2::theme_minimal()
 }
 
+
 #' Refitting Generalized Linear Models
 #'
 #' @description `r lifecycle::badge('experimental')`
-#'  `refit_glm()` is used to refit generalized linear models, and must be
+#'  `update_glm()` is used to refit generalized linear models, and must be
 #'  preceded by `restrict_coef()`.
 #'
 #' @param x Object of class restricted or of class smooth
@@ -438,18 +522,28 @@ autoplot.smooth <- function(object, ...){
 #' @return Object of class GLM
 #'
 #' @export
-refit_glm <- function(x){
+update_glm <- function(x){
 
   if( !inherits(x, c("restricted", "smooth")) ) {
     stop("Input must be of class restricted or of class smooth", call. = FALSE)
   }
 
   lst_call <- as.list(x$model_call)
-  lst <- list(formula = x$formula_restricted, data = x$data_restricted, offset = NULL)
+  lst <- list(formula = x$formula_restricted, data = x$data_restricted,
+              offset = NULL)
   y <- eval(as.call(modifyList(lst_call, lst)))
   y$call$formula <- lst$formula
   y$call$data <- quote(df_new)
 
+  offweights <- NULL
+  if ( !is.null(lst_call$weights) ) {
+    offweights <- append(offweights, as.character(lst_call$weights))
+  }
+
+  if ( !is.null(lst_call$offset) ) {
+    offweights <- append(offweights, as.character(lst_call$offset)[2])
+  }
+
   if ( inherits(x, "smooth")) {
     attr(y, "new_rf") <- x[["new_rf"]]
     attr(y, "class") <- append(class(y), "refitsmooth")
@@ -460,32 +554,18 @@ refit_glm <- function(x){
     attr(y, "class") <- append(class(y), "refitrestricted")
   }
 
+  rf <- x$rating_factors
+  rf2 <- unique(rf$risk_factor[rf$risk_factor != "(Intercept)"])
+
+  attr(y, "new_col_nm") <- x$new_col_nm
+  attr(y, "old_col_nm") <- x$old_col_nm
+  attr(y, "rf") <- rf2
+  attr(y, "mgd_smt") <- x$mgd_smt
+  attr(y, "mgd_rst") <- x$mgd_rst
+  attr(y, "offweights") <- offweights
   y
 }
 
-#' Get data from refitted Generalized Linear Model
-#'
-#' @description `r lifecycle::badge('experimental')`
-#'  `get_data()` is used to get data from refitted generalized linear models, and must be
-#'  preceded by `refit_glm()`.
-#'
-#' @param x Object of class refitsmooth or of class refitrestricted
-#'
-#' @author Martin Haringa
-#'
-#' @return data.frame
-#'
-#' @export
-get_data <- function(x){
-  if( !inherits(x, c("refitsmooth", "refitrestricted")) ) {
-    stop("Input must be of class refitsmooth or of class refitrestricted", call. = FALSE)
-  }
-  xdf <- x$data
-  xdf[!names(xdf) %in% c("breaks_min", "breaks_max",
-                         "start_oc", "end_oc",
-                         "start_", "end_",
-                         "avg_", "risk_factor")]
-}
 
 
 
